
#include "Arduino.h"
#include "nrf.h"
#include "mbed.h"

#define INPUT_PIN 3 // entrée de mesure
#define OUTPUT_PIN 4 // sortie à relier à l'entrée (pour tester)
#define PPI_COUNT_CH 10 // canaux PPI
#define PPI_CAPTURE_CH 11

float frequence;
mbed::PwmOut pwm(digitalPinToPinName(OUTPUT_PIN));
uint8_t prescaler; // prescaler pour TIMER3
uint32_t compare; // CC[0] pour TIMER 3

const uint8_t pins[14] = {0,0,11,12,15,13,14,23,21,27,2,1,8,13};
const uint8_t ports[14] = {0,0,1,1,1,1,1,0,0,0,1,1,1,0};

// mesure de fréquence sur une durée égale à 1/(16e6)*2^(prescaler)*compare
void mesure_frequence() {
  NRF_GPIOTE->CONFIG[0] = 1 | (pins[INPUT_PIN]<<8) | (ports[INPUT_PIN]<<13) | (1<<16); // evènement rising edge sur INPUT_PIN
  
  NRF_TIMER4->TASKS_STOP = 1;
  NRF_TIMER4->MODE = 1; // counter mode
  NRF_TIMER4->BITMODE = 3; // 32 bits
  NRF_TIMER4->TASKS_CLEAR = 1;
  
  
  NRF_PPI->CH[PPI_COUNT_CH].EEP = (uint32_t)&NRF_GPIOTE->EVENTS_IN[0];
  NRF_PPI->CH[PPI_COUNT_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_COUNT; // incrémenter le compteur du timer 4 lorsque INPUT_PIN passe de 0 à 1
  NRF_PPI->CHEN |= (1<< ((uint32_t)PPI_COUNT_CH));
  NRF_PPI->CHENSET |= (1<< ((uint32_t)PPI_COUNT_CH));
   
  NRF_TIMER3->TASKS_STOP = 1;
  NRF_TIMER3->MODE = 0; // timer mode
  NRF_TIMER3->BITMODE = 3; // 32 bits
  NRF_TIMER3->TASKS_CLEAR = 1;
  NRF_TIMER3->PRESCALER = prescaler;
  NRF_TIMER3->CC[0] = compare;
  NRF_TIMER3->SHORTS = 1<<8; // event COMPARE[0] -> task STOP
  NRF_TIMER3->EVENTS_COMPARE[0] = 0;
  NRF_PPI->CH[PPI_CAPTURE_CH].EEP = (uint32_t)&NRF_TIMER3->EVENTS_COMPARE[0];
  NRF_PPI->CH[PPI_CAPTURE_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_CAPTURE[0]; // copie la valeur du compteur dans CC[0]
  NRF_PPI->FORK[PPI_CAPTURE_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_STOP;
  NRF_PPI->CHEN |= (1<< ((uint32_t)PPI_CAPTURE_CH));
  NRF_PPI->CHENSET |= (1<< ((uint32_t)PPI_CAPTURE_CH));
  
  NRF_TIMER4->TASKS_START = 1;
  NRF_TIMER3->TASKS_START = 1;

  float duree = compare*1.0/16*1e-3*pow(2,prescaler); // en ms
  
  rtos::ThisThread::sleep_for(duree);
  while (NRF_TIMER3->EVENTS_COMPARE[0]==0) {;} // attente de la fin du comptage
  NRF_TIMER3->TASKS_STOP = 1;
  frequence = (NRF_TIMER4->CC[0])/duree*1000;
}

void setup() {
  Serial.begin(115200);
  pinMode(INPUT_PIN,INPUT);
  pwm.period_us(10);
  pwm.write(0.5);
  prescaler = 0; // une incrémentation de TIMER3 toutes les 1/16 microsecondes
  compare = 16000000; // durée de 1.0 s
}

void loop() {
   mesure_frequence();                                                                                                     
   Serial.print(frequence); Serial.println(" Hz");
  
}

    	               